CREATE TABLE process_list
    (
      transaction_id INT IDENTITY(1, 1)
                         PRIMARY KEY ,
      status_flag BIT ,
      last_update DATETIME2 ,
      transaction_type INT ,
      transaction_desc VARCHAR(25)
    );
GO

CREATE INDEX IX_giant_index_process_list
ON dbo.process_list (transaction_id, status_flag);
GO

-- show that there is no fragmentation on the newly created table
SELECT  OBJECT_SCHEMA_NAME(o.object_id) AS schema_name,
        o.name,
        i.name,
        ddps.reserved_page_count AS reserved_pages_empty,
		ddps.reserved_page_count * 8.0 as "size in KB"
FROM    sys.objects AS o
        JOIN sys.dm_db_partition_stats AS ddps ON ddps.object_id = o.object_id
        JOIN sys.indexes AS i ON i.object_id = ddps.object_id AND i.index_id = ddps.index_id
WHERE   o.name = 'process_list' ;

/* Random string generation code taken from:

http://social.technet.microsoft.com/wiki/contents/articles/21196.t-sql-random-string.aspx

*/

/* insert another 1000 rows */
DECLARE @string_max_length TINYINT = 25;
DECLARE @i	int = 0


DECLARE @StringMaxLen int = 25

-- This is where things differ from http://www.brentozar.com/archive/2014/11/index-fragmentation-matter/ as I had to use a different random string generator because master.dbo.spt_values
-- does not exist in SQL Azure Database and I inserted 100000 records
INSERT  INTO process_list (status_flag, last_update, transaction_type, transaction_desc)
SELECT TOP (100000)
	0 ,
    GETUTCDATE() ,
	len(LEFT (REPLACE(CAST (NEWID () AS NVARCHAR(MAX)),'-','') , ABS (CHECKSUM (NEWID ())) % @StringMaxLen + 1)) % 4 + 1,
    LEFT (REPLACE(CAST (NEWID () AS NVARCHAR(MAX)),'-','') , ABS (CHECKSUM (NEWID ())) % @StringMaxLen + 1)
FROM SYS.OBJECTS A
CROSS JOIN SYS.OBJECTS B
CROSS JOIN SYS.OBJECTS C

-- make sure that all of the records have been included
select count(*) as record_count from process_list

-- show index size after 100000 records have been inserted 
SELECT  OBJECT_SCHEMA_NAME(o.object_id) AS schema_name,
        o.name,
        i.name,
        ddps.reserved_page_count AS reserved_pages_100000_records,
		ddps.reserved_page_count * 8.0 as "size in KB"
FROM    sys.objects AS o
        JOIN sys.dm_db_partition_stats AS ddps ON ddps.object_id = o.object_id
        JOIN sys.indexes AS i ON i.object_id = ddps.object_id AND i.index_id = ddps.index_id
WHERE   o.name = 'process_list' ;

-- Change the status of 70%ish of transaction to TRUE
UPDATE process_list
SET     status_flag = 1
WHERE   transaction_id % 10 < 7 ;

/* Get rid of processed rows */
DELETE FROM process_list
WHERE status_flag = 1 ;

-- show index size after fragmenting delete
SELECT  OBJECT_SCHEMA_NAME(o.object_id) AS schema_name,
        o.name,
        i.name,
        ddps.reserved_page_count AS reserved_pages_after_delete,
		ddps.reserved_page_count * 8.0 as "size in KB"
FROM    sys.objects AS o
        JOIN sys.dm_db_partition_stats AS ddps ON ddps.object_id = o.object_id
        JOIN sys.indexes AS i ON i.object_id = ddps.object_id AND i.index_id = ddps.index_id
WHERE   o.name = 'process_list' ;
GO

-- rebuild the indexes
ALTER INDEX ALL ON dbo.process_list REBUILD ;

-- show the index size after the rebuild
SELECT  OBJECT_SCHEMA_NAME(o.object_id) AS schema_name,
        o.name,
        i.name,
        ddps.reserved_page_count AS reserved_pages_after_rebuild,
		ddps.reserved_page_count * 8.0 as "size in KB"
FROM    sys.objects AS o
        JOIN sys.dm_db_partition_stats AS ddps ON ddps.object_id = o.object_id
        JOIN sys.indexes AS i ON i.object_id = ddps.object_id AND i.index_id = ddps.index_id
WHERE   o.name = 'process_list' ;
GO

